gBINDING_HEADER_sRaidFrames = "sRaidFrames"

local L = AceLibrary("AceLocale-2.2"):new("sRaidFrames")
local surface = AceLibrary("Surface-1.0") 

sRaidFrames.options = {
	type = "group",
	args = {
		lock = {
			name = L["Lock"],
			type = "toggle",
			desc = L["Lock/Unlock the raid frames"],
			get = function()
				return sRaidFrames.opt.Locked
			end,
			set = function(locked)
				sRaidFrames.opt.Locked = locked
				if not locked then
					sRaidFrames:S("ShowGroupTitles", true)
					for _,f in pairs(sRaidFrames.groupframes) do
						sRaidFrames:UpdateTitleVisibility(f.header)
					end
				end
			end,
			map = {[false] = L["Unlocked"], [true] = L["Locked"]},
			disabled = function() return InCombatLockdown() end,
		},

		health = {
			name = L["Health text"],
			type = "text",
			desc = L["Set health display type"],
			get = function()
				return sRaidFrames.opt.HealthFormat
			end,
			set = function(value)
				sRaidFrames.opt.HealthFormat = value
			end,
			validate = {["curmax"] = L["Current and max health"], ["deficit"] = L["Health deficit"], ["percent"] = L["Health percentage"], ["current"] = L["Current health"], ["none"] = L["Hide health text"]},
		},

		bufffilter = {
			name = L["Buff filter"],
			type = "group",
			desc = L["Set buff filter"],
			args = {
				add = {
					name = L["Add buff"],
					type = "text",
					desc = L["Add a buff"],
					get = false,
					set = function(value)
						if value and value ~= "" and not sRaidFrames.opt.BuffFilter[value] then
							sRaidFrames.opt.BuffFilter[value] = true
							sRaidFrames:chatUpdateBuffMenu()
						end
					end,
					usage = L["<name of buff>"],
				},
			},
			disabled = function() return (sRaidFrames.opt.BuffType == "debuffs") end,
		},

		titles = {
			name = L["Show group titles"],
			type = "toggle",
			desc = L["Toggle display of titles above each group frame"],
			get = function()
				return sRaidFrames.opt.ShowGroupTitles
			end,
			set = function(value)
				sRaidFrames:S("ShowGroupTitles", value)
				for _,f in pairs(sRaidFrames.groupframes) do
					sRaidFrames:UpdateTitleVisibility(f.header)
				end
			end,
			disabled = function() return not sRaidFrames.opt.Locked or InCombatLockdown() end,
		},

		bufftype = {
			name = L["Buff/Debuff visibility"],
			type = "text",
			desc = L["Show buffs or debuffs on the raid frames"],
			get = function()
				return sRaidFrames.opt.BuffType
			end,
			set = "chatBuffType",
			validate = {["buffs"] = L["Only buffs"], ["debuffs"] = L["Only debuffs"], ["buffsifnotdebuffed"] = L["Buffs if not debuffed"]},
		},

		powerfilter = {
			name = L["Power type visiblity"],
			type = "group",
			desc = L["Toggle the display of certain power types (Mana, Rage, Energy)"],
			args = {
				mana = {
					name = L["Mana"],
					type = "toggle",
					desc = L["Toggle the display of mana bars"],
					get = function()
						return sRaidFrames.opt.PowerFilter[0]
					end,
					set = function(value)
						sRaidFrames.opt.PowerFilter[0] = value
						sRaidFrames:UpdateAllUnits()
					end,
					map = {[false] = L["hidden"], [true] = L["shown"]},
				},
				energy = {
					name = L["Energy & Focus"],
					type = "toggle",
					desc = L["Toggle the display of energy and focus bars"],
					get = function()
						return sRaidFrames.opt.PowerFilter[2]
					end,
					set = function(value)
						sRaidFrames.opt.PowerFilter[2] = value
						sRaidFrames.opt.PowerFilter[3] = value
						sRaidFrames:UpdateAllUnits()
					end,
					map = {[false] = L["hidden"], [true] = L["shown"]},
				},
				rage = {
					name = L["Rage"],
					type = "toggle",
					desc = L["Toggle the display of rage bars"],
					get = function()
						return sRaidFrames.opt.PowerFilter[1]
					end,
					set = function(value)
						sRaidFrames.opt.PowerFilter[1] = value
						sRaidFrames:UpdateAllUnits()
					end,
					map = {[false] = L["hidden"], [true] = L["shown"]},
				},
			},
		},

		filterdebuffs = {
			name = L["Filter dispellable debuffs"],
			type = "toggle",
			desc = L["Toggle display of dispellable debuffs only"],
			get = function()
				return sRaidFrames.opt.ShowOnlyDispellable
			end,
			set = "chatToggleDispellable",
		},
		
		invert = {
			name = L["Invert health bars"],
			type = "toggle",
			desc = L["Invert the growth of the health bars"],
			get = function()
				return sRaidFrames.opt.Invert
			end,
			set = function(value)
				sRaidFrames.opt.Invert = value
				sRaidFrames:UpdateAllUnits()
			end,
		},

		texture = {
			name = L["Bar textures"],
			type = "text",
			desc = L["Set the texture used on health and mana bars"],
			get = function()
				return sRaidFrames.opt.Texture
			end,
			set = "chatTexture",
			validate = surface:List(),
		},

		scale = {
			name = L["Scale"],
			type = "range",
			desc = L["Set the scale of the raid frames"],
			min = 0.1,
			max = 3.0,
			step = 0.05,
			get = function()
				return sRaidFrames.opt.Scale
			end,
			set = "chatScale",
			disabled = function() return InCombatLockdown() end,
		},

		positioning = {
			name = L["Positioning"],
			type = "group",
			desc = L["Set the positioning of the raid frames"],
			args = {
				reset = {
					name = L["Reset position"],
					type = "execute",
					desc = L["Reset the position of sRaidFrames"],
					func = "ResetPosition"
				},
				predefined = {
					name = L["Predefined position"],
					type = "text",
					desc = L["Set a predefined position for the raid frames"],
					get = function() return nil end,
					set = "chatSetPosition",
					validate = {["ctra"] = L["CT_RaidAssist"], ["horizontal"] = L["Horizontal"], ["vertical"] = L["Vertical"]},
				},
			},
			disabled = function() return InCombatLockdown() end,
		},
		
		layout = {
			name = L["Layout"],
			type = "text",
			desc = L["Set the layout of the frames based on presets"],
			get = function()
				return sRaidFrames.opt.Layout
			end,
			set = function(value)
				sRaidFrames.opt.Layout = value
				sRaidFrames:SetLayout(value)
				sRaidFrames:ApplyLayout()
			end,
			validate = {},
		},
		
		colors = {
			name = L["Colors"],
			type = "group",
			desc = L["Set the diffirent colors of the raid frames"],
			args = {
				background = {
					type = "color",
					name = L["Background color"],
					desc = L["Change the background color"],
					get = function()
						local s = sRaidFrames.opt.BackgroundColor
						return s.r, s.g, s.b, s.a
					end,
					set = "chatBackgroundColor",
					hasAlpha = true,
					order = 100,
				},
				border = {
					type = "color",
					name = L["Border color"],
					desc = L["Change the border color"],
					get = function()
						local s = sRaidFrames.opt.BorderColor
						return s.r, s.g, s.b, s.a
					end,
					set = "chatBorderColor",
					hasAlpha = true,
					disabled = function() return not sRaidFrames.opt.Border end,
					order = 100,
				},
				
				{
					type = "header",
					name = L["Debuff colors"],
					order = 200,
				},
				
				magic = {
					type = "color",
					name = L["Magic debuff"],
					desc = L["Change the background color for magic debuffs"],
					get = function()
						local s = sRaidFrames.opt.DebuffColors["Magic"]
						return s.r, s.g, s.b, s.a
					end,
					set = function(r, g, b, a)
						sRaidFrames.opt.DebuffColors["Magic"] = {r = r, g = g, b = b, a = a}
					end,
					hasAlpha = true,
					order = 201,
				},
				curse = {
					type = "color",
					name = L["Curse debuff"],
					desc = L["Change the background color for curse debuffs"],
					get = function()
						local s = sRaidFrames.opt.DebuffColors["Curse"]
						return s.r, s.g, s.b, s.a
					end,
					set = function(r, g, b, a)
						sRaidFrames.opt.DebuffColors["Curse"] = {r = r, g = g, b = b, a = a}
					end,
					hasAlpha = true,
					order = 201,
				},
				poison = {
					type = "color",
					name = L["Poison debuff"],
					desc = L["Change the background color for poison debuffs"],
					get = function()
						local s = sRaidFrames.opt.DebuffColors["Poison"]
						return s.r, s.g, s.b, s.a
					end,
					set = function(r, g, b, a)
						sRaidFrames.opt.DebuffColors["Poison"] = {r = r, g = g, b = b, a = a}
					end,
					hasAlpha = true,
					order = 201,
				},
				disease = {
					type = "color",
					name = L["Disease debuff"],
					desc = L["Change the background color for disease debuffs"],
					get = function()
						local s = sRaidFrames.opt.DebuffColors["Disease"]
						return s.r, s.g, s.b, s.a
					end,
					set = function(r, g, b, a)
						sRaidFrames.opt.DebuffColors["Disease"] = {r = r, g = g, b = b, a = a}
					end,
					hasAlpha = true,
					order = 201,
				},
			},
		},

		tooltips = {
			name = L["Tooltip display"],
			type = "group",
			desc = L["Determine when a tooltip is displayed"],
			args = {
				units = {
					name = L["Unit tooltips"],
					type = "text",
					desc = L["Determine when a tooltip is displayed"],
					get = function() return sRaidFrames.opt.UnitTooltipMethod end,
					set = function(value)
						sRaidFrames:S("UnitTooltipMethod", value)
					end,
					validate = {["never"] = L["Never"], ["notincombat"] = L["Only when not in combat"], ["always"] = L["Always"]},			
				},
				buffs = {
					name = L["Buff tooltips"],
					type = "text",
					desc = L["Determine when a tooltip is displayed"],
					get = function() return sRaidFrames.opt.BuffTooltipMethod end,
					set = function(value)
						sRaidFrames:S("BuffTooltipMethod", value)
					end,
					validate = {["never"] = L["Never"], ["notincombat"] = L["Only when not in combat"], ["always"] = L["Always"]},
				},
				debuffs = {
					name = L["Debuff tooltips"],
					type = "text",
					desc = L["Determine when a tooltip is displayed"],
					get = function() return sRaidFrames.opt.DebuffTooltipMethod end,
					set = function(value)
						sRaidFrames:S("DebuffTooltipMethod", value)
					end,
					validate = {["never"] = L["Never"], ["notincombat"] = L["Only when not in combat"], ["always"] = L["Always"]},
				},
			},
		},

		aggro = {
				name = L["Highlight units with aggro"],
				type = "toggle",
				desc = L["Turn the border of units who have aggro red"],
				get = function() return sRaidFrames.opt.AggroCheck end,
				set = function(value)
					sRaidFrames.opt.AggroCheck = value
				end,
				disabled = function() return not sRaidFrames:GetLayout().HasBorders end,
		},
		
		highlight = {
				name = L["Highlight targetted unit"],
				type = "toggle",
				desc = L["Turn the border of unit you are targetting orange"],
				get = function() return sRaidFrames.opt.HighlightTarget end,
				set = function(value)
					sRaidFrames:S("HighlightTarget", value)
				end,
				disabled = function() return not sRaidFrames:GetLayout().HasBorders end,
		},
		
		heals = {
			name = L["Heal tracking"],
			type = "group",
			desc = L["Options for heal tracking"],
			args = {
				enable = {
						name = L["Enable heal tracking"],
						type = "toggle",
						desc = L["Turn the border of units getting heals green"],
						get = function() return sRaidFrames.opt.HighlightHeals end,
						set = function(value)
							sRaidFrames:S("HighlightHeals", value)
						end,

				},
				self = {
						name = L["Show self heals"],
						type = "toggle",
						desc = L["Show heals originating from yourself"],
						get = function() return sRaidFrames.opt.HighlightHealsSelf end,
						set = function(value)
							sRaidFrames:S("HighlightHealsSelf", value)
						end,
						disabled = function() return not sRaidFrames.opt.HighlightHeals end,
						
				},
			},
			disabled = function() return not sRaidFrames:GetLayout().HasBorders or not CastCommLib end,
		},

		range = {
			name = L["Range"],
			type = "group",
			desc = L["Options for range checks"],
			args = {
				enable = {
					name = L["Enable range check"],
					type = "toggle",
					desc = L["Enable range checking"],
					get = function() return sRaidFrames.opt.RangeCheck end,
					set = function(value)
						sRaidFrames.opt.RangeCheck = value
						if not value then
							for unit in pairs(sRaidFrames.frames) do
								sRaidFrames.frames[unit]:SetAlpha(1)
							end
						end
					end,
					order = 1,
				},
				alpha = {
					name = L["Alpha"],
					type = "range",
					desc = L["The alpha level for units who are out of range"],
					get = function() return sRaidFrames.opt.RangeAlpha end,
					set = function(value)
						sRaidFrames.opt.RangeAlpha = value
					end,
					min  = 0,
					max  = 1,
					step = 0.1,
					disabled = function() return not sRaidFrames.opt.RangeCheck end,
				},
				frequency = {
					name = L["Frequency"],
					type = "range",
					desc = L["The interval between which range checks are performed"],
					get = function() return sRaidFrames.opt.RangeFrequency end,
					set = function(value)
						sRaidFrames.opt.RangeFrequency = value
						sRaidFrames:UpdateRangeFrequency()
					end,
					min  = 0.2,
					max  = 2.0,
					step = 0.1,
					disabled = function() return not sRaidFrames.opt.RangeCheck end,
				},
				limit = {
					name = L["Range"],
					type = "text",
					desc = L["The range at which a unit is considered out of range"],
					get = function() return tostring(sRaidFrames.opt.RangeLimit) end,
					set = function(value)
						sRaidFrames.opt.RangeLimit = tonumber(value)
					end,
					validate = {},
					disabled = function() return not sRaidFrames.opt.RangeCheck end,
				}
			},
		},

		sets = {
			name = "Filter sets",
			type = "group",
			desc = "Toggle the display of certain Groups/Classes",
			args = {
				reset = {
					name = "Reset to default",
					type = "execute",
					desc = "Reset to default sets",
					func = function()
						
					end,
				},
			},				
		},

		growth = {
			name = L["Growth"],
			type = "text",
			desc = L["Set the growth of the raid frames"],
			get = function()
				return sRaidFrames.opt.Growth
			end,
			set = function(value)
				sRaidFrames.opt.Growth = value
				sRaidFrames:SetGrowth()
			end,
			validate = {["up"] = L["Up"], ["down"] = L["Down"], ["left"] = L["Left"], ["right"] = L["Right"]},
			disabled = function() return InCombatLockdown() end,
		},

		spacing = {
			name = L["Frame Spacing"],
			type = "range",
			desc = L["Set the spacing between each of the raid frames"],
			min = -5,
			max = 5,
			step = 1,
			get = function()
				return sRaidFrames.opt.Spacing
			end,
			set = function(s)
				sRaidFrames.opt.Spacing = s
				sRaidFrames:SetSpacing()
			end,
			disabled = function() return InCombatLockdown() end,
		},
	}
}

function sRaidFrames:chatUpdateBuffMenu()
	self.options.args.bufffilter.args["remove"] = {}
	self.options.args.bufffilter.args["remove"].type = 'group'
	self.options.args.bufffilter.args["remove"].name = L["Remove buff"]
	self.options.args.bufffilter.args["remove"].desc = L["Remove buffs from the list"]
	self.options.args.bufffilter.args["remove"].args = {}
	local i = 1
	for buff in pairs(self.opt.BuffFilter) do
		local buffName = buff -- Odd hack, don't know
		self.options.args.bufffilter.args["remove"].args["buff" .. i] = {}
		self.options.args.bufffilter.args["remove"].args["buff" .. i].type = 'execute'
		self.options.args.bufffilter.args["remove"].args["buff" .. i].name = buffName
		self.options.args.bufffilter.args["remove"].args["buff" .. i].desc = string.format(L["Remove %s from the buff list"],buffName)
		self.options.args.bufffilter.args["remove"].args["buff" .. i].func = function() self.opt.BuffFilter[buffName] = nil self:chatUpdateBuffMenu()  end
		i = i + 1
	end
end

function sRaidFrames:chatBuffType(value)
	self:S("BuffType", value)

	self:UpdateAllUnits()
end

function sRaidFrames:chatToggleDispellable(value)
	self:S("ShowOnlyDispellable", value)

	self:UpdateAllUnits()
end

function sRaidFrames:chatTexture(texture)
	self.opt.Texture = texture

	for _, f in pairs(self.frames) do
		f.hpbar:SetStatusBarTexture(surface:Fetch(self.opt.Texture))
		f.mpbar:SetStatusBarTexture(surface:Fetch(self.opt.Texture))
	end
end

function sRaidFrames:chatScale(t)
	self:S("Scale", t)

	self.master:SetScale(t)
end

function sRaidFrames:chatBackgroundColor(r, g, b, a)
	self:S("BackgroundColor", {r = r, g = g, b = b, a = a})

	-- Need to do this, since someone might be debuffed, and so will need a diffirent background color
	self:UpdateAllUnits()
end

function sRaidFrames:chatBorderColor(r, g, b, a)
	self:S("BorderColor", {r = r, g = g, b = b, a = a})

	for k,f in pairs(self.frames) do
		f:SetBackdropBorderColor(r, g, b, a)
	end
end

function sRaidFrames:chatSetPosition(layout)
	self:PositionLayout(layout, self.groupframes[1]:GetLeft(), self.groupframes[1]:GetTop()-UIParent:GetTop())
end


function sRaidFrames:UpdateRangeLimitOptions()
	self.options.args["range"].args["limit"].validate = {}
	for r, _ in pairs(self.RangeChecks) do
		self.options.args["range"].args["limit"].validate[tostring(r)] = string.format(L["%d yards"], r)
	end
end 